# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import hmac
from warnings import warn
from abc import (
    ABC,
    abstractmethod,
)
from typing import (
    Any,
    Callable,
    Dict,
    Mapping,
    Type,
    TypeVar,
    Union,
    Optional
)
from functools import wraps

import cryptography.hazmat.primitives.serialization as serialization_module
from cryptography.exceptions import InvalidSignature
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.asymmetric.rsa import (
    rsa_crt_dmp1,
    rsa_crt_dmq1,
    rsa_crt_iqmp,
    rsa_recover_prime_factors,
    RSAPrivateKey,
    RSAPrivateNumbers,
    RSAPublicKey,
    RSAPublicNumbers,
)
from cryptography.hazmat.primitives.hashes import HashAlgorithm

from .exceptions import (
    MalformedJWKError,
    UnsupportedKeyTypeError,
)
from .utils import (
    b64encode,
    b64decode,
    uint_b64encode,
    uint_b64decode,
)

_AJWK = TypeVar("_AJWK", bound="AbstractJWKBase")
_T = TypeVar("_T")


class AbstractJWKBase(ABC):

    @abstractmethod
    def get_kty(self) -> str:
        pass  # pragma: no cover

    @abstractmethod
    def get_kid(self) -> str:
        pass  # pragma: no cover

    @abstractmethod
    def is_sign_key(self) -> bool:
        pass  # pragma: no cover

    @abstractmethod
    def sign(self, message: bytes, **options) -> bytes:
        pass  # pragma: no cover

    @abstractmethod
    def verify(self, message: bytes, signature: bytes, **options) -> bool:
        pass  # pragma: no cover

    @abstractmethod
    def to_dict(self, public_only: bool = True) -> Dict[str, str]:
        pass  # pragma: no cover

    @classmethod
    @abstractmethod
    def from_dict(cls: Type[_AJWK], dct: Dict[str, object]) -> _AJWK:
        pass  # pragma: no cover


class OctetJWK(AbstractJWKBase):

    def __init__(self, key: bytes, kid=None, **options) -> None:
        super(AbstractJWKBase, self).__init__()
        self.key = key
        self.kid = kid

        optnames = {'use', 'key_ops', 'alg', 'x5u', 'x5c', 'x5t', 'x5t#s256'}
        self.options = {k: v for k, v in options.items() if k in optnames}

    def get_kty(self):
        return 'oct'

    def get_kid(self):
        return self.kid

    def is_sign_key(self) -> bool:
        return True

    def _get_signer(self, options) -> Callable[[bytes, bytes], bytes]:
        return options['signer']

    def sign(self, message: bytes, **options) -> bytes:
        signer = self._get_signer(options)
        return signer(message, self.key)

    def verify(self, message: bytes, signature: bytes, **options) -> bool:
        signer = self._get_signer(options)
        return hmac.compare_digest(signature, signer(message, self.key))

    def to_dict(self, public_only=True):
        dct = {
            'kty': 'oct',
            'k': b64encode(self.key),
        }
        dct.update(self.options)
        if self.kid:
            dct['kid'] = self.kid
        return dct

    @classmethod
    def from_dict(cls, dct):
        try:
            return cls(b64decode(dct['k']), **dct)
        except KeyError as why:
            raise MalformedJWKError('k is required') from why


class RSAJWK(AbstractJWKBase):
    """
    https://tools.ietf.org/html/rfc7518.html#section-6.3.1
    """

    def __init__(self, keyobj: Union[RSAPrivateKey, RSAPublicKey],
                 **options) -> None:
        super(AbstractJWKBase, self).__init__()
        self.keyobj = keyobj

        optnames = {'use', 'key_ops', 'alg', 'kid',
                    'x5u', 'x5c', 'x5t', 'x5t#s256', }
        self.options = {k: v for k, v in options.items() if k in optnames}

    def is_sign_key(self) -> bool:
        return isinstance(self.keyobj, RSAPrivateKey)

    def _get_hash_fun(self, options) -> Callable[[], HashAlgorithm]:
        return options['hash_fun']

    def _get_padding(self, options) -> padding.AsymmetricPadding:
        try:
            return options['padding']
        except KeyError:
            warn('you should not use RSAJWK.verify/sign without jwa '
                 'intermiediary, used legacy padding')
            return padding.PKCS1v15()

    def sign(self, message: bytes, **options) -> bytes:
        if isinstance(self.keyobj, RSAPublicKey):
            raise ValueError("Requires a private key.")
        hash_fun = self._get_hash_fun(options)
        _padding = self._get_padding(options)
        return self.keyobj.sign(message, _padding, hash_fun())

    def verify(self, message: bytes, signature: bytes, **options) -> bool:
        hash_fun = self._get_hash_fun(options)
        _padding = self._get_padding(options)
        if isinstance(self.keyobj, RSAPrivateKey):
            pubkey = self.keyobj.public_key()
        else:
            pubkey = self.keyobj
        try:
            pubkey.verify(signature, message, _padding, hash_fun())
            return True
        except InvalidSignature:
            return False

    def get_kty(self):
        return 'RSA'

    def get_kid(self):
        return self.options.get('kid')

    def to_dict(self, public_only=True):
        dct = {
            'kty': 'RSA',
        }
        dct.update(self.options)

        if isinstance(self.keyobj, RSAPrivateKey):
            priv_numbers = self.keyobj.private_numbers()
            pub_numbers = priv_numbers.public_numbers
            dct.update({
                'e': uint_b64encode(pub_numbers.e),
                'n': uint_b64encode(pub_numbers.n),
            })
            if not public_only:
                dct.update({
                    'e': uint_b64encode(pub_numbers.e),
                    'n': uint_b64encode(pub_numbers.n),
                    'd': uint_b64encode(priv_numbers.d),
                    'p': uint_b64encode(priv_numbers.p),
                    'q': uint_b64encode(priv_numbers.q),
                    'dp': uint_b64encode(priv_numbers.dmp1),
                    'dq': uint_b64encode(priv_numbers.dmq1),
                    'qi': uint_b64encode(priv_numbers.iqmp),
                })
            return dct
        pub_numbers = self.keyobj.public_numbers()
        dct.update({
            'e': uint_b64encode(pub_numbers.e),
            'n': uint_b64encode(pub_numbers.n),
        })
        return dct

    @classmethod
    def from_dict(cls, dct):
        if 'oth' in dct:
            raise UnsupportedKeyTypeError(
                'RSA keys with multiples primes are not supported')

        try:
            e = uint_b64decode(dct['e'])
            n = uint_b64decode(dct['n'])
        except KeyError as why:
            raise MalformedJWKError('e and n are required') from why
        pub_numbers = RSAPublicNumbers(e, n)
        if 'd' not in dct:
            return cls(
                pub_numbers.public_key(backend=default_backend()), **dct)
        d = uint_b64decode(dct['d'])

        privparams = {'p', 'q', 'dp', 'dq', 'qi'}
        product = set(dct.keys()) & privparams
        if len(product) == 0:
            p, q = rsa_recover_prime_factors(n, e, d)
            priv_numbers = RSAPrivateNumbers(
                d=d,
                p=p,
                q=q,
                dmp1=rsa_crt_dmp1(d, p),
                dmq1=rsa_crt_dmq1(d, q),
                iqmp=rsa_crt_iqmp(p, q),
                public_numbers=pub_numbers)
        elif product == privparams:
            priv_numbers = RSAPrivateNumbers(
                d=d,
                p=uint_b64decode(dct['p']),
                q=uint_b64decode(dct['q']),
                dmp1=uint_b64decode(dct['dp']),
                dmq1=uint_b64decode(dct['dq']),
                iqmp=uint_b64decode(dct['qi']),
                public_numbers=pub_numbers)
        else:
            # If the producer includes any of the other private key parameters,
            # then all of the others MUST be present, with the exception of
            # "oth", which MUST only be present when more than two prime
            # factors were used.
            raise MalformedJWKError(
                'p, q, dp, dq, qi MUST be present or'
                'all of them MUST be absent')
        return cls(priv_numbers.private_key(backend=default_backend()), **dct)


def supported_key_types() -> Dict[str, Type[AbstractJWKBase]]:
    return {
        'oct': OctetJWK,
        'RSA': RSAJWK,
    }


def jwk_from_dict(dct: Mapping[str, Any]) -> AbstractJWKBase:
    if not isinstance(dct, dict):  # pragma: no cover
        raise TypeError('dct must be a dict')
    if 'kty' not in dct:
        raise MalformedJWKError('kty MUST be present')

    supported = supported_key_types()
    kty = dct['kty']
    if kty not in supported:
        raise UnsupportedKeyTypeError('unsupported key type: {}'.format(kty))
    return supported[kty].from_dict(dct)


PublicKeyLoaderT = Union[str, Callable[[bytes, object], object]]
PrivateKeyLoaderT = Union[
    str,
    Callable[[bytes, Optional[str], object], object]]
_Loader = TypeVar("_Loader", PublicKeyLoaderT, PrivateKeyLoaderT)
_C = TypeVar("_C", bound=Callable[..., Any])


# The above LoaderTs should actually not be Union, and this function should be
# typed something like this. But, this will lose any kwargs from the typing
# information. Probably needs: https://github.com/python/mypy/issues/3157
# (func: Callable[[bytes, _Loader], _T])
#   -> Callable[[bytes, Union[str, _Loader]], _T]
def jwk_from_bytes_argument_conversion(func: _C) -> _C:
    if not ('private' in func.__name__ or 'public' in func.__name__):
        raise Exception("the wrapped function must have either public"
                        " or private in it's name")

    @wraps(func)
    def wrapper(content, loader, **kwargs):
        # now convert it to a Callable if it's a string
        if isinstance(loader, str):
            loader = getattr(serialization_module, loader)

        if kwargs.get('options') is None:
            kwargs['options'] = {}

        return func(content, loader, **kwargs)
    return wrapper  # type: ignore[return-value]


@jwk_from_bytes_argument_conversion
def jwk_from_private_bytes(
    content: bytes,
    private_loader: PrivateKeyLoaderT,
    *,
    password: Optional[str] = None,
    backend: Optional[object] = None,
    options: Optional[Mapping[str, object]] = None,
) -> AbstractJWKBase:
    """This function is meant to be called from jwk_from_bytes"""
    if options is None:
        options = {}
    try:
        privkey = private_loader(content, password, backend)  # type: ignore[operator]  # noqa: E501
        if isinstance(privkey, RSAPrivateKey):
            return RSAJWK(privkey, **options)
        raise UnsupportedKeyTypeError('unsupported key type')
    except ValueError as ex:
        raise UnsupportedKeyTypeError('this is probably a public key') from ex


@jwk_from_bytes_argument_conversion
def jwk_from_public_bytes(
    content: bytes,
    public_loader: PublicKeyLoaderT,
    *,
    backend: Optional[object] = None,
    options: Optional[Mapping[str, object]] = None
) -> AbstractJWKBase:
    """This function is meant to be called from jwk_from_bytes"""
    if options is None:
        options = {}
    try:
        pubkey = public_loader(content, backend)  # type: ignore[operator]
        if isinstance(pubkey, RSAPublicKey):
            return RSAJWK(pubkey, **options)
        raise UnsupportedKeyTypeError(
            'unsupported key type')  # pragma: no cover
    except ValueError as why:
        raise UnsupportedKeyTypeError('could not deserialize') from why


def jwk_from_bytes(
    content: bytes,
    private_loader: PrivateKeyLoaderT,
    public_loader: PublicKeyLoaderT,
    *,
    private_password: Optional[str] = None,
    backend: Optional[object] = None,
    options: Optional[Mapping[str, object]] = None,
) -> AbstractJWKBase:
    try:
        return jwk_from_private_bytes(
            content,
            private_loader,
            password=private_password,
            backend=backend,
            options=options,
        )
    except UnsupportedKeyTypeError:
        return jwk_from_public_bytes(
            content,
            public_loader,
            backend=backend,
            options=options,
        )


def jwk_from_pem(
    pem_content: bytes,
    private_password: Optional[str] = None,
    options: Optional[Mapping[str, object]] = None,
) -> AbstractJWKBase:
    return jwk_from_bytes(
        pem_content,
        private_loader='load_pem_private_key',
        public_loader='load_pem_public_key',
        private_password=private_password,
        backend=None,
        options=options,
    )


def jwk_from_der(
    der_content: bytes,
    private_password: Optional[str] = None,
    options: Optional[Mapping[str, object]] = None,
) -> AbstractJWKBase:
    return jwk_from_bytes(
        der_content,
        private_loader='load_der_private_key',
        public_loader='load_der_public_key',
        private_password=private_password,
        backend=None,
        options=options,
    )



"""
Created on 2022-01-17
@auth:刘飞
@description:自定义分页
"""
from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response


class MyPageNumberPagination(PageNumberPagination):
    page_size = 10  # 数字，页面显示的记录条数，不设置的就默认setting配置的全局PAGE_SIZE值
    page_size_query_param = 'size'  # 字符串，请求设置页面大小的参数名称，默认是None
    max_page_size = 10000  # 最大允许请求的页面大小，仅在page_size_query_param被设置时有效

    def get_paginated_response(self, data):
        return Response({
            'total': self.page.paginator.count,
            # 'list': add_order(data),
            'list': data,
        })


def add_order(lst):
    for i, j in enumerate(lst):
        j['order'] = i + 1
    return lst



# encoding: utf-8
"""
@project: djangoModel->user_bank_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户银行卡服务
@created_time: 2023/6/8 16:20
"""
from django.core.paginator import Paginator, EmptyPage
from django.db.models import F
from xj_user.models import UserBankCards
from ..utils.custom_tool import format_params_handle, force_transform_type


class UserBankCardsService():
    @staticmethod
    def get_bank_card(params=None, detail_id=None, allow_user_list=None):
        allow_user_list = [i for i in allow_user_list if i] if allow_user_list else None
        # 查看详情
        detail_id, err = force_transform_type(variable=detail_id, var_type="int")
        if detail_id:
            return UserBankCards.objects.filter(id=detail_id).values().first(), None
        else:
            params, err = force_transform_type(variable=params, var_type="dict", default={})
            size, err = force_transform_type(variable=params.get("size"), var_type="int", default=10)
            page, err = force_transform_type(variable=params.get("page"), var_type="int", default=1)
            sort = params.get("sort")
            filter_params = format_params_handle(
                param_dict=params,
                filter_filed_list=["id", "user_id", "bank_card_num", "open_account_bank", "opening_branch",
                                   "is_default", "user_id_list"],
                split_list=["user_id_list"],
                alias_dict={"user_id_list": "user_id__in"},
            )
            # 排序字段
            allow_sort = ["id", "-id", "sort", "-sort"]
            sort = sort if sort in allow_sort else "-id"
            user_bank_card_set = UserBankCards.objects
            if not allow_user_list is None and isinstance(allow_user_list, list):  # 筛选可以访问的列表
                user_bank_card_set = user_bank_card_set.filter(id__in=allow_user_list)
            cards_obj = user_bank_card_set.filter(**filter_params).annotate(
                username=F("user__username"),
                fullname=F("user__fullname"),
                nickname=F("user__nickname"), ).order_by(sort).values()
            count = cards_obj.count()
            try:
                page_set = Paginator(cards_obj, size).get_page(page)
            except EmptyPage:
                return {'count': count, "page": page, "size": size, "list": []}, None

            return {'count': count, "page": page, "size": size, "list": list(page_set.object_list)}, None

    @staticmethod
    def add(params=None):
        if params is None:
            params = {}
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["user_id|int", "bank_card_num", "open_account_bank", "opening_branch", "is_default|int",
                               "remark",
                               "ext|dict"]
        )
        try:
            cards_obj = UserBankCards.objects.create(**filter_params)
            return {"id": cards_obj.id}, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def edit(pk=None, update_params=None):
        if update_params is None:
            update_params = {}
        filter_params = format_params_handle(
            param_dict=update_params,
            filter_filed_list=["bank_card_num", "open_account_bank", "opening_branch", "is_default|int", "remark",
                               "ext|dict"]
        )
        if not pk or not filter_params:
            return None, "没有可修改的数据"
        try:
            cards_obj = UserBankCards.objects.filter(id=pk)
            if not cards_obj:
                return None, "没有可修改的数据"
            cards_obj.update(**filter_params)

            return None, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def delete(pk=None):
        if not pk:
            return None, "参数错误"
        cards_obj = UserBankCards.objects.filter(id=pk)
        if not cards_obj:
            return None, None
        try:
            cards_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None



# -*- encoding:utf-8 -*-

import datetime
import json


class JTransformType:
    @staticmethod
    # 强制转换类型数据
    def to(variable=None, var_type: str = None, default=None, **kwargs):
        """
        强制转换类型，转换规则如下：
        1.变量为空使用默认值返回。认为是空值判断。
        2.转换类型为空则，返回输入变量。认为不需要进行类型转换。
        3.无法转换则返回默认值，转换失败，数据存在问题则返回默认值。
        :param default: 数据为空或无法转换的时候返回默认值
        :param variable: 变量
        :param var_type: 转换类型 str|int|bool|float|dict|list|date
        :return: transform_variable, is_err
        """
        if variable is None:  # 为空时候，则返回默认值
            return default, None
        if not var_type:  # 转换类型为空，则不进行转换
            return variable, None
        # 强制类型转换
        try:
            if var_type == "str":
                return variable if isinstance(variable, str) else str(variable), None
            elif var_type == "int":
                return variable if isinstance(variable, int) else int(variable), None
            elif var_type == "bool":
                if variable in (None, '', [], (), {}):
                    return None, None
                if variable in (True, False):
                    return bool(variable), None
                if variable in ('t', 'True', '1'):
                    return True, None
                if variable in ('f', 'False', '0'):
                    return False, None
                return None, "'" + str(variable) + "'" + "不是一个有效的布尔类型数据"
            elif var_type == "float":
                return variable if isinstance(variable, float) else float(variable), None
            elif var_type == "only_dict":
                # 不包括列表
                variable = JTransformType.__parse_json(variable)
                if isinstance(variable, dict):
                    return variable, None
                return default, str(variable) + "无法转换为json格式"
            elif var_type == "dict":
                # 字典包括列表字典
                variable = JTransformType.__parse_json(variable)
                if isinstance(variable, dict) or isinstance(variable, list):
                    return variable, None
                return default, str(variable) + "无法转换为json格式"
            elif var_type == "list":
                variable = JTransformType.__parse_json(variable)
                if isinstance(variable, list):
                    return variable, None
                return default, str(variable) + "无法转换为list格式"
            elif var_type == "list_int":
                variable = JTransformType.__parse_json(variable)
                result = []
                if isinstance(variable, list):
                    for i in variable:
                        # 过滤掉非int类型
                        i, is_pass = JTransformType.to(variable=i, var_type="int")
                        if is_pass:
                            continue
                        result.append(i)
                    return result, None
                return default, str(variable) + "无法转换为list格式"
            elif var_type == "date":
                format_map = {
                    '%Y-%m-%d': "%Y-%m-%d 00:00:00",
                    '%Y-%m-%d %H': '%Y-%m-%d %H:00:00',
                    '%Y-%m-%d %H:%M': '%Y-%m-%d %H:%M:00',
                    '%Y-%m-%d %H:%M:%S': '%Y-%m-%d %H:%M:%S'
                }
                for format_input, format_output in format_map.items():
                    try:
                        return datetime.datetime.strptime(variable, format_input).strftime(format_output), None
                    except ValueError:
                        pass
                raise ValueError
            else:  # 如果指派的类型不存在，默认值非空则返回默认值，否则原样返回
                return variable if default is None else default, str(var_type) + "不是一个有效的转换类型"
        except TypeError:
            return default, "'" + str(variable) + "'" + "不是一个有效的" + str(var_type) + "类型数据"
        except ValueError:
            return default, "'" + str(variable) + "'" + "不是一个有效的" + str(var_type) + "类型数据"

    @staticmethod
    # json 结果集返回
    def __parse_json(result):
        if not result is None:
            if type(result) is str:
                try:
                    result = json.loads(result.replace("'", '"').replace('\\r', "").replace('\\n', "").replace('\\t', "").replace('\\t', ""))
                except Exception as e:
                    return result
            if type(result) is list:
                for index, value in enumerate(result):
                    result[index] = JTransformType.__parse_json(value)
            if type(result) is dict:
                for k, v in result.items():
                    result[k] = JTransformType.__parse_json(v)
        return result








import sys
py2 = sys.version_info < (3, 0)


if py2:
    str_type = unicode

    def u(s):
        if not isinstance(s, unicode):
            s = unicode(s, "utf-8")
        return s

else:
    str_type = str

    def u(s):
        return s



from django.test import TestCase
from ..services.user_list_service import UserListService

class TestUserList(TestCase):
    def setUp(self):
        # 初始化测试数据
        self.test_params = {
            "page": 1,
            "size": 10,
            "sort": "-id",
            "username": "testuser",
            "email": "test@example.com",
            "phone": "1234567890",
            "nickname": "testnick",
            "fullname": "Test User",
            "is_using": True,
            "user_level": 1
        }

    def test_user_list_with_params(self):
        # 测试带参数的 user_list 方法
        result, err = UserListService.user_list(params=self.test_params)
        print("> TestUserList: test_user_list_with_params:", result, err)
        self.assertEqual(err, None)
        self.assertIn('list', result)
        self.assertIn('count', result)
        self.assertIn('size', result)
        self.assertIn('page', result)

    def test_user_list_without_params(self):
        # 测试不带参数的 user_list 方法
        result, err = UserListService.user_list()
        print("> TestUserList: test_user_list_without_params:", result, err)
        self.assertEqual(err, None)
        self.assertIn('list', result)
        self.assertIn('count', result)
        self.assertIn('size', result)
        self.assertIn('page', result)

    def test_user_list_with_invalid_sort(self):
        # 测试带无效排序参数的 user_list 方法
        invalid_params = self.test_params.copy()
        invalid_params["sort"] = "invalid_sort"
        result, err = UserListService.user_list(params=invalid_params)
        print("> TestUserList: test_user_list_with_invalid_sort:", result, err)
        self.assertEqual(err, None)
        self.assertIn('list', result)
        self.assertIn('count', result)
        self.assertIn('size', result)
        self.assertIn('page', result)

    def test_user_list_with_empty_search(self):
        # 测试带空搜索参数的 user_list 方法
        empty_search_params = {
            "page": 1,
            "size": 10,
            "sort": "-id"
        }
        result, err = UserListService.user_list(params=empty_search_params)
        print("> TestUserList: test_user_list_with_empty_search:", result, err)
        self.assertEqual(err, None)
        self.assertIn('list', result)
        self.assertIn('count', result)
        self.assertIn('size', result)
        self.assertIn('page', result)


#!/usr/local/bin/python
# -*- coding: utf-8 -*-

'''
A utility to translate Mandarin to English. The data file is from
the CEDICT project. We keep it compressed for size and speed.

Using this has substantial (1-2 second) init time for reading the
dictionary. This is spent the first time this module is used (it
is free to import).

Note that this is a prototype. It is specific to Python 3. To bring
this code to the quality level of the rest of the library, it would
need to be backported to Python 2.

In addition, it'd be nice if `all_phrase_translations` handled both
traditional and simplified elegantly.
'''

import gzip
import os.path
import re
import string

import collections


def Tree():
    return collections.defaultdict(Tree)


dictionaries = None  # Used for single word lookup
trees = None  # Used for parsing.


def _add_to_tree(tree, word, meaning):
    '''
    We build word search trees, where we walk down
    the letters of a word. For example:
      你 Good
      你好 Hello
    Would build the tree
         你
        /  \
      You   好
             \
           Hello
    '''
    if len(word) == 0:
        tree[''] = meaning
    else:
        _add_to_tree(tree[word[0]], word[1:], meaning)


def init():
    '''
    Load in the Chinese-English dictionary. This takes 1-2 seconds. It
    is done when the other functions are used, but this is public since
    preloading sometimes makes sense.
    '''
    global dictionaries, trees

    dictionaries = {
        'traditional': {},
        'simplified': {}
    }

    trees = {
        'traditional': Tree(),
        'simplified': Tree()
    }

    lines = gzip.open(
        os.path.join(os.path.dirname(__file__), "cedict.txt.gz"),
        mode='rt',
        encoding='utf-8'
    )
    exp = re.compile("^([^ ]+) ([^ ]+) \[(.*)\] /(.+)/")
    parsed_lines = (exp.match(line).groups()
                    for line in lines
                    if line[0] != '#')

    for traditional, simplified, pinyin, meaning in parsed_lines:
        meaning = meaning.split('/')
        dictionaries['traditional'][traditional] = meaning
        dictionaries['simplified'][simplified] = meaning
        _add_to_tree(trees['traditional'], traditional, meaning)
        _add_to_tree(trees['simplified'], simplified, meaning)


def translate_word(word, dictionary=['simplified']):
    '''
    Return the set of translations for a single character or word, if
    available.
    '''
    if not dictionaries:
        init()
    for d in dictionary:
        if word in dictionaries[d]:
            return dictionaries[d][word]
    return None


def _words_at_the_beginning(word, tree, prefix=""):
    '''
    We return all portions of the tree corresponding to the beginning
    of `word`. This is used recursively, so we pass the prefix so we
    can return meaningful words+translations.
    '''
    l = []
    if "" in tree:
        l.append([prefix, tree[""]])
    if len(word) > 0 and word[0] in tree:
        l.extend(_words_at_the_beginning(
            word[1:],
            tree[word[0]],
            prefix=prefix+word[0]
        ))
    return l


def all_phrase_translations(phrase):
    '''
    Return the set of translations for all possible words in a full
    phrase. Chinese is sometimes ambiguous. We do not attempt to
    disambiguate, or handle unknown letters especially well. Full
    parsing is left to upstream logic.
    '''
    if not trees:
        init()
    phrase = phrase.split(string.whitespace)
    for word in phrase:
        for x in range(len(word)):
            for translation in _words_at_the_beginning(
                    word[x+1:],
                    trees['simplified'][word[x]],
                    prefix=word[x]):
                yield translation



# Generated by Django 4.2.17 on 2026-01-22 21:50

from django.db import migrations
import xj_user.utils.j_ulid_field


class Migration(migrations.Migration):

    dependencies = [
        ('xj_user', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='baseinfo',
            name='uuid',
            field=xj_user.utils.j_ulid_field.JULIDField(blank=True, db_index=True, default='1kfjzgggevye', max_length=32, null=True, unique=True, verbose_name='UUID'),
        ),
    ]



from typing import List

from django.db import models
from django.conf import settings


# 在模型中写如下代码
# models.Model.to_json = JDjangoSimpleApi.serialize_model
# models.query.QuerySet.to_json = JDjangoSimpleApi.serialize_queryset
# models.query.RawQuerySet.to_json = JDjangoSimpleApi.serialize_queryset


# from django_simple_api.utils import string_convert, do_nothing
# 对django_simple_api库的serialize文件提纯出来，可使models拥有to_json()转为字符串方法
class JDjangoSimpleApi:
    @staticmethod
    def serialize_model(self: models.Model, excludes: List[str] = None) -> dict:
        """
        模型序列化，会根据 select_related 和 prefetch_related 关联查询的结果进行序列化，可以在查询时使用 only、defer 来筛选序列化的字段。
        它不会自做主张的去查询数据库，只用你查询出来的结果，成功避免了 N+1 查询问题。

        # See：
        https://aber.sh/articles/A-new-idea-of-serializing-Django-model/
        """
        excludes = excludes or []
        serialized = set()

        if getattr(settings, "DSA_SERIALIZE_TO_CAMELCASE", False):
            to_camel_case_func = string_convert
        else:
            to_camel_case_func = do_nothing

        def _serialize_model(model) -> dict:

            # 当 model 存在一对一字段时，会陷入循环，使用闭包的自由变量存储已序列化的 model，
            # 在第二次循环到该 model 时直接返回 model.pk，不再循环。
            nonlocal serialized
            if model in serialized:
                return model.pk
            else:
                serialized.add(model)

            # 当 model 存在一对一或一对多字段，且该字段的值为 None 时，直接返回空{}，否则会报错。
            if model is None:
                return {}

            result = {
                to_camel_case_func(name): _serialize_model(foreign_key)
                for name, foreign_key in model.__dict__["_state"]
                .__dict__.get("fields_cache", {})
                .items()
            }

            buried_fields = getattr(model, "buried_fields", [])

            for name, value in model.__dict__.items():

                # 敏感字段不需要序列化
                if name in buried_fields:
                    continue

                # 私有属性不需要序列化
                if name.startswith("_"):
                    continue

                result[to_camel_case_func(name)] = value

            for name, queryset in model.__dict__.get(
                "_prefetched_objects_cache", {}
            ).items():
                result[to_camel_case_func(name)] = [_serialize_model(model) for model in queryset]  # type: ignore

            return result

        results = _serialize_model(self)

        # 剔除排斥的字段
        for field_name in excludes:
            del results[to_camel_case_func(field_name)]

        return results

    @staticmethod
    def serialize_queryset(self: models.QuerySet, excludes: List[str] = None) -> List[dict]:
        return [JSimpleSerialize.serialize_model(model, excludes) for model in self]


def do_nothing(x):
    return x


def string_convert(string: str):
    """
    将连字符格式转成小驼峰格式
    example: username -> userName
    """

    char_list = string.split("_")

    # 首个单词不转换
    for index, char in enumerate(char_list[1:], start=1):
        char_list[index] = char.capitalize()

    return "".join(char_list)


# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from datetime import datetime, timezone
from typing import AbstractSet, Any, Dict, Optional

from .utils import (
    get_time_from_int,
)
from .exceptions import (
    JWSEncodeError,
    JWSDecodeError,
    JWTEncodeError,
    JWTDecodeError,
)
from .jwk import AbstractJWKBase
from .jws import JWS


class JWT:

    def __init__(self):
        self._jws = JWS()

    def encode(self, payload: Dict[str, Any],
               key: Optional[AbstractJWKBase] = None, alg='HS256',
               optional_headers: Optional[Dict[str, str]] = None) -> str:
        if not isinstance(self, JWT):  # pragma: no cover
            # https://github.com/GehirnInc/python-jwt/issues/15
            raise RuntimeError(
                'encode must be called on a jwt.JWT() instance. '
                'Do jwt.JWT().encode(...)')
        if not isinstance(payload, dict):  # pragma: no cover
            raise TypeError('payload must be a dict')
        if not (key is None
                or isinstance(key, AbstractJWKBase)):  # pragma: no cover
            raise TypeError(
                'key must be an instance of a class implements '
                'jwt.AbstractJWKBase')
        if not (optional_headers is None
                or isinstance(optional_headers, dict)):  # pragma: no cover
            raise TypeError('optional_headers must be a dict')

        try:
            message = json.dumps(payload).encode('utf-8')
        except ValueError as why:
            raise JWTEncodeError(
                'payload must be able to be encoded to JSON') from why

        optional_headers = optional_headers and optional_headers.copy() or {}
        optional_headers['typ'] = 'JWT'
        try:
            return self._jws.encode(message, key, alg, optional_headers)
        except JWSEncodeError as why:
            raise JWTEncodeError('failed to encode to JWT') from why

    def decode(self, message: str, key: Optional[AbstractJWKBase] = None,
               do_verify=True, algorithms: Optional[AbstractSet[str]] = None,
               do_time_check: bool = True) -> Dict[str, Any]:
        if not isinstance(self, JWT):  # pragma: no cover
            # https://github.com/GehirnInc/python-jwt/issues/15
            raise RuntimeError(
                'decode must be called on a jwt.JWT() instance. '
                'Do jwt.JWT().decode(...)')
        if not isinstance(message, str):  # pragma: no cover
            raise TypeError('message must be a str')
        if not (key is None
                or isinstance(key, AbstractJWKBase)):  # pragma: no cover
            raise TypeError(
                'key must be an instance of a class implements '
                'jwt.AbstractJWKBase')

        # utc now with timezone
        now = datetime.now(timezone.utc)
        try:
            message_bin = self._jws.decode(message, key, do_verify, algorithms)
        except JWSDecodeError as why:
            raise JWTDecodeError('failed to decode JWT') from why
        try:
            payload = json.loads(message_bin.decode('utf-8'))
        except ValueError as why:
            raise JWTDecodeError(
                'a payload of the JWT is not valid JSON') from why

        # The "exp" (expiration time) claim identifies the expiration time on
        # or after which the JWT MUST NOT be accepted for processing.
        if 'exp' in payload and do_time_check:
            try:
                exp = get_time_from_int(payload['exp'])
            except TypeError:
                raise JWTDecodeError("Invalid Expired value")
            if now >= exp:
                raise JWTDecodeError("JWT Expired")

        # The "nbf" (not before) claim identifies the time before which the JWT
        # MUST NOT be accepted for processing.
        if 'nbf' in payload and do_time_check:
            try:
                nbf = get_time_from_int(payload['nbf'])
            except TypeError:
                raise JWTDecodeError('Invalid "Not valid yet" value')
            if now < nbf:
                raise JWTDecodeError("JWT Not valid yet")

        return payload



# -*- coding: utf-8 -*-
"""
Created on 2022-04-20
@author:刘飞
@description:
一些公共处理方法
此方法尽量不要引用其他文件的方法，容易出现混合引用问题
"""
import json
import os
import uuid
import logging
import hashlib
import datetime
import string
import secrets
from django.core.cache import cache, caches
from django.conf import settings

log = logging.getLogger()


def get_file_path(instance, filename):
    """文件保存路径"""
    folder = instance.__class__.__name__.lower() + datetime.datetime.now().strftime("/%Y/%m/%d")
    ext = filename.split('.')[-1]
    filename = "%s.%s" % (uuid.uuid4(), ext)
    return os.path.join(folder, filename)


def get_file_path_by_name(folder_name, filename):
    """
    传递名称返回文件相对路径
    """
    folder = folder_name.lower() + datetime.datetime.now().strftime("/%Y/%m/%d")  # 获取
    ext = filename.split('.')
    full_path = os.path.join(settings.MEDIA_ROOT, folder, "%s.%s" % (ext[0], ext[1]))  # 获取文件绝对路径
    flag = True
    while flag:  # 如果有相同名称文件处理[在服务器和内存中都查找]
        if os.path.isfile(full_path) or cache.detail(f'{folder}/{filename}'):
            ext[0] = ext[0] + get_rand_str(6)
            filename = "%s.%s" % (ext[0], ext[1])
            full_path = os.path.join(settings.MEDIA_ROOT, folder, "%s.%s" % (ext[0], ext[1]))
        else:
            flag = False
    return "%s/%s" % (folder, filename)


def delete_file_path(file_path):
    """
    根据文件路径删除
    """
    file_path = str(file_path).lstrip('/media/')
    path = os.path.join(settings.MEDIA_ROOT, file_path)
    if os.path.isfile(path):
        os.remove(path)


def delete_file_diff(old_list, new_list):
    """
    根据列表不同删除
    旧的里面有，新的里面没有，即视为删除图片
    """
    [delete_file_path(p) for p in set(old_list).difference(set(new_list))]


def save_file(file_name):
    """
    文件保存，根据文件相对路径拼接后从内存中写入到服务器上
    不管是不是新上传都走进去,不在内存中会跳过
    前后端分离注意：这里传进来的file_name是没有/media/前缀的
    """
    # 这里再处理一下
    file_name = str(file_name).lstrip('/media/')
    file = cache.detail(file_name)
    if file:
        filename = os.path.join(settings.MEDIA_ROOT, file_name)  # 资源绝对路径获取
        pos = filename.rfind("/")
        filepath = filename[:pos]
        if not os.path.exists(filepath):  # 资源目录
            os.makedirs(filepath)
        with open(filename, 'wb') as f:
            for chunk in file.chunks():  # 分块写入文件
                f.write(chunk)
        cache.delete(file_name)
    else:
        log.info(f'需要保存的文件在内存中不存在！{file_name}')


def md5(password, salt):
    """密码加密"""
    m = hashlib.md5(bytes(password, encoding='utf-8'))
    # m.update(bytes(salt, encoding='utf-8'))
    m = hashlib.md5(bytes(m.hexdigest() + salt, encoding='utf-8'))
    return m.hexdigest()


def get_rand_str(n):
    """
    生成随机长度的字母数字字符串
    """
    alphabet = string.ascii_letters + string.digits
    res = ''.join(secrets.choice(alphabet) for i in range(n))
    return res


def parse_json(self, json_data):
    """
    格式化json数据，避免json返回时出现字符串json情况
    :param self:
    :param json_data:
    :return:
    """
    for k, v in json_data.items():
        if not type(v) is dict:
            try:
                json_data[k] = json.loads(v)
            except Exception as e:
                json_data[k] = v
        else:
            json_data[k] = self.parse_json(v)
    return json_data



# encoding: utf-8
"""
@project: djangoModel->wechet_login
@author:
@created_time: 2022/7/14 10:55
"""
# 微信登录方法
from logging import getLogger

from django.http import HttpResponse
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from ..services.login_service import LoginService
from ..services.user_main_service import UserMainService
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper, flow_service_wrapper
from ..utils.write_to_log import write_to_log
from ..utils.user_wrapper import user_authentication_wrapper

logger = getLogger('log')


class UserLoginMain(APIView):

    @api_view(['POST'])
    @request_params_wrapper
    @flow_service_wrapper
    def login_main(self, *args, request_params=None, **kwargs):
        response = HttpResponse()
        response.status_code = 200
        if request_params is None:
            request_params = {}
        # logger.info('---用户登录参数打印：' + str(request_params) + '---')

        data, err = UserMainService.login_integration_interface(request_params)
        # print('> login_main: data, err:', data, err)
        if err:
            if isinstance(err, dict) and err.get("error"):
                content = util_response(data=err['wechat_data'], msg=err['msg'], err=int(err['error']))
            else:
                content = util_response(err=6001, msg=err)

            response.content = content
            return response

        # 判斷賬號
        # if data["user_info"]["is_using"] != 1:  # (志聪)语法问题，没有检查键存在直接访问，多次报错待改 by Sieyoo at 20231219
        if data.get('user_info', None) and data['user_info'].get('is_using', None) == 0:
            response.content = util_response(err=6003, msg="账号已停用，禁止登录，请联系平台管理员处理。")
            return response
        content = util_response(data=data)
        response['Authorization'] = data.get("token", "")

        response.content = content
        return response

    @api_view(['POST'])
    @user_authentication_wrapper
    @request_params_wrapper
    def bind_phone(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        user_id = user_info.get("user_id")
        bind_phone, err = LoginService.bind_phone(user_id, params.get("phone", ""))
        if err is None:
            return util_response(data=bind_phone)
        return util_response(err=47767, msg=err)

    @api_view(['POST'])
    @user_authentication_wrapper
    @request_params_wrapper
    def wechat_unbinding(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        user_id = user_info.get("user_id")
        wechat_unbinding, err = LoginService.wechat_unbinding(params)
        if err is None:
            return util_response(data=wechat_unbinding)
        return util_response(err=47767, msg=err)

    @api_view(['POST'])
    @user_authentication_wrapper
    @request_params_wrapper
    def secondary_authorization(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        user_id = user_info.get("user_id")
        params.setdefault("user_id", user_id)  # 用户ID
        empower, err = LoginService.secondary_authorization(params)
        if err is None:
            return util_response(data=empower)
        return util_response(err=47767, msg=err)

    # @api_view(['POST'])
    # @request_params_wrapper
    # def send(self, *args, request_params=None, **kwargs):
    #     params = request_params
    #     parameter = "{'account':%s,'pwd':%s}" % (123, 456)
    #     bind_phone, err = LoginService.template_send("applet", "register", )
    #     if err is None:
    #         return util_response(data=bind_phone)
    #     return util_response(err=47767, msg=err)

    @api_view(['POST'])
    @request_params_wrapper
    @flow_service_wrapper
    def register(self, *args, request_params=None, **kwargs):
        response = HttpResponse()
        response.status_code = 200
        if request_params is None:
            request_params = {}

        platform_code = request_params.get("platform_code", None)
        username = request_params.get('account', '')
        password = str(request_params.get('password', ''))
        nickname = request_params.get('nickname', '')
        phone = request_params.get('phone', '')
        data, err = LoginService.register(
            username=username,
            password=password,
            nickname=nickname,
            phone=phone,
            platform_code=platform_code,
        )
        if err is None:
            return util_response(data=data)
        return util_response(err=47767, msg=err)



# encoding: utf-8
"""
@project: Animated-All-Star->observer
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 监听者模式抽象类继承
@created_time: 2022/11/15 8:20
"""
from __future__ import annotations

from abc import ABC, abstractmethod
from typing import List


class Observer(ABC):
    @abstractmethod
    def update(self, subject: Subject) -> None:
        pass


class Subject(ABC):
    _observers: List[Observer] = []

    @abstractmethod
    def attach(self, observer: Observer) -> None:
        pass

    @abstractmethod
    def detach(self, observer: Observer) -> None:
        pass

    @abstractmethod
    def notify(self, subject=None) -> None:
        pass



# _*_coding:utf-8_*_
import sys

from django.views.decorators.http import require_http_methods
from rest_framework.views import APIView

from ..services.user_relate_service import UserRelateToUserService

if 'xj_role' in sys.modules:
    from xj_role.services.role_service import RoleService
    from xj_role.services.user_group_service import UserGroupService
from ..models import Platform, PlatformsToUsers
from ..services.user_detail_info_service import DetailInfoService
from ..services.user_service import UserService
from ..utils.custom_tool import request_params_wrapper, format_params_handle
from ..utils.model_handle import util_response
from ..utils.user_wrapper import user_authentication_force_wrapper


# 管理员添加用户
class UserAdd(APIView):

    @require_http_methods(['POST'])
    @user_authentication_force_wrapper
    @request_params_wrapper
    def add(self, *args, request_params=None, **kwargs):
        if request_params is None:
            request_params = {}
        if not request_params.get("platform_code", None):
            return util_response(err=1001, msg="平台码不能为空")
        platform_set = Platform.objects.filter(platform_code=request_params.get("platform_code", None)).first()
        if not platform_set:
            return util_response(err=1001, msg="平台不存在")
        # 获取角色和部门的id列表
        user_role_list = request_params.pop('user_role_list', None)
        user_group_list = request_params.pop('user_group_list', None)
        belong_list = request_params.pop('belong_list', None)

        # TODO 针对劳务通系统做的个人用户绑定企业用户功能
        relate_key = request_params.pop('relate_key', None)

        # 进行用户添加
        data, err = UserService.user_add(request_params)
        if err:
            return util_response(err=1001, msg=err)

        # 联动添加用户详细信息
        user_id = data.get("id")
        if user_id:
            detail_params = format_params_handle(
                param_dict=request_params,
                remove_filed_list=["account", "password", "fullname", "nickname"]
            )
            detail_params.setdefault("user_id", user_id)
            detail_params.setdefault("real_name", "系统分配用户")
            detail_add_data, err = DetailInfoService.create_or_update_detail(detail_params)
        print(f"user_id: {user_id}")
        platforms_users = PlatformsToUsers.objects.create(**{
            "platform_id": platform_set.id,
            "user_id": user_id
        })
        if not platforms_users:
            return "平台写入失败"

        # 绑定用户的角色和组织
        if user_group_list and user_id:
            UserGroupService.user_bind_groups(user_id, user_group_list)
        if user_role_list and user_id:
            RoleService.bind_user_role(user_id, user_role_list)

        # TODO 针对劳务通系统做的个人用户绑定企业用户功能
        if belong_list and user_id:
            UserRelateToUserService.laowu_bind_bxtx_relate(
                {
                    "user_id_list": belong_list,
                    "with_user_id": user_id,
                    "relate_key": relate_key
                })

        return util_response(data=data)



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import hashlib
import hmac
from typing import (
    Any,
    Dict,
    Callable,
    Optional,
)

from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.hashes import (
    SHA256,
    SHA384,
    SHA512,
)

from .exceptions import InvalidKeyTypeError
from .jwk import AbstractJWKBase


def std_hash_by_alg(alg: str) -> Callable[[bytes], object]:
    if alg.endswith('S256'):
        return hashlib.sha256
    if alg.endswith('S384'):
        return hashlib.sha384
    if alg.endswith('S512'):
        return hashlib.sha512
    raise ValueError('{} is not supported'.format(alg))


class AbstractSigningAlgorithm:

    def sign(self, message: bytes, key: Optional[AbstractJWKBase]) -> bytes:
        raise NotImplementedError()  # pragma: no cover

    def verify(self, message: bytes, key: Optional[AbstractJWKBase],
               signature: bytes) -> bool:
        raise NotImplementedError()  # pragma: no cover


class NoneAlgorithm(AbstractSigningAlgorithm):

    def sign(self, message: bytes, key: Optional[AbstractJWKBase]) -> bytes:
        return b''

    def verify(self, message: bytes, key: Optional[AbstractJWKBase],
               signature: bytes) -> bool:
        return hmac.compare_digest(signature, b'')


none = NoneAlgorithm()


class HMACAlgorithm(AbstractSigningAlgorithm):

    def __init__(self, hash_fun: Callable[[], object]) -> None:
        self.hash_fun = hash_fun

    def _check_key(self, key: Optional[AbstractJWKBase]) -> AbstractJWKBase:
        if not key or key.get_kty() != 'oct':
            raise InvalidKeyTypeError('Octet key is required')
        return key

    def _sign(self, message: bytes, key: bytes) -> bytes:
        return hmac.new(key, message, self.hash_fun).digest()

    def sign(self, message: bytes, key: Optional[AbstractJWKBase]) -> bytes:
        key = self._check_key(key)
        return key.sign(message, signer=self._sign)

    def verify(self, message: bytes, key: Optional[AbstractJWKBase],
               signature: bytes) -> bool:
        key = self._check_key(key)
        return key.verify(message, signature, signer=self._sign)


HS256 = HMACAlgorithm(hashlib.sha256)
HS384 = HMACAlgorithm(hashlib.sha384)
HS512 = HMACAlgorithm(hashlib.sha512)


class RSAAlgorithm(AbstractSigningAlgorithm):

    def __init__(self, hash_fun: object) -> None:
        self.hash_fun = hash_fun

    def _check_key(
        self,
        key: Optional[AbstractJWKBase],
        must_sign_key: bool = False,
    ) -> AbstractJWKBase:
        if not key or key.get_kty() != 'RSA':
            raise InvalidKeyTypeError('RSA key is required')
        if must_sign_key and not key.is_sign_key():
            raise InvalidKeyTypeError(
                'a RSA private key is required, but passed is RSA public key')
        return key

    def sign(self, message: bytes, key: Optional[AbstractJWKBase]) -> bytes:
        key = self._check_key(key, must_sign_key=True)
        return key.sign(message, hash_fun=self.hash_fun,
                        padding=padding.PKCS1v15())

    def verify(
        self,
        message: bytes,
        key: Optional[AbstractJWKBase],
        signature: bytes,
    ) -> bool:
        key = self._check_key(key)
        return key.verify(message, signature, hash_fun=self.hash_fun,
                          padding=padding.PKCS1v15())


RS256 = RSAAlgorithm(SHA256)
RS384 = RSAAlgorithm(SHA384)
RS512 = RSAAlgorithm(SHA512)


class PSSRSAAlgorithm(AbstractSigningAlgorithm):
    def __init__(self, hash_fun: Callable[[], Any]) -> None:
        self.hash_fun = hash_fun

    def _check_key(
        self,
        key: Optional[AbstractJWKBase],
        must_sign_key: bool = False,
    ) -> AbstractJWKBase:
        if not key or key.get_kty() != 'RSA':
            raise InvalidKeyTypeError('RSA key is required')
        if must_sign_key and not key.is_sign_key():
            raise InvalidKeyTypeError(
                'a RSA private key is required, but passed is RSA public key')
        return key

    def sign(self, message: bytes, key: Optional[AbstractJWKBase]) -> bytes:
        key = self._check_key(key, must_sign_key=True)
        return key.sign(
            message,
            hash_fun=self.hash_fun,
            padding=padding.PSS(  # type: ignore[no-untyped-call]
                mgf=padding.MGF1(self.hash_fun()),
                salt_length=self.hash_fun().digest_size,
            ),
        )

    def verify(
        self,
        message: bytes,
        key: Optional[AbstractJWKBase],
        signature: bytes
    ) -> bool:
        key = self._check_key(key)
        return key.verify(
            message,
            signature,
            hash_fun=self.hash_fun,
            padding=padding.PSS(  # type: ignore[no-untyped-call]
                mgf=padding.MGF1(self.hash_fun()),
                salt_length=self.hash_fun().digest_size,
            ),
        )


PS256 = PSSRSAAlgorithm(SHA256)
PS384 = PSSRSAAlgorithm(SHA384)
PS512 = PSSRSAAlgorithm(SHA512)


def supported_signing_algorithms() -> Dict[str, AbstractSigningAlgorithm]:
    # NOTE(yosida95): exclude vulnerable 'none' algorithm by default.
    return {
        'HS256': HS256,
        'HS384': HS384,
        'HS512': HS512,
        'RS256': RS256,
        'RS384': RS384,
        'RS512': RS512,
        'PS256': PS256,
        'PS384': PS384,
        'PS512': PS512,
    }



"""
Created on 2022-05-20
@author:刘飞
@description:用户验证装饰器，为单个请求方式做验证
"""
from rest_framework import exceptions
from xj_user.services.user_service import UserService


def authentication_wrapper(func):
    def inner(*args, **kwargs):
        request = args[1]
        token = request.headers.get('Authorization', None)
        user, error_text = UserService.check_token(token)
        if error_text:
            raise exceptions.AuthenticationFailed(error_text)

        request.user = user
        response = func(*args, **kwargs)
        return response

    return inner



# _*_coding:utf-8_*_

from django.core.paginator import Paginator, EmptyPage
from django.db.models import Q, F

from ..models import ExtendField, DetailInfo, BaseInfo
from ..utils.custom_tool import format_params_handle
from ..utils.j_join_list import JJoinList

class UserListService:
    # 显式列出 BaseInfo 模型的字段
    base_fields = [
        "id", "uuid", "user_no", "username", "nickname", "fullname", "phone", "email",
        "avatar", "user_type", "user_vip", "user_level", "user_info", "register_time",
        "register_ip", "privacies", "is_auth", "is_delete", "is_using"
    ]

    # 显式列出 DetailInfo 模型的字段
    detail_fields = [
        "user_id", "real_name", "sex", "birth", "tags", "signature", "avatar", "cover",
        "id_card_type", "id_card_no", "language", "region_code", "more",
        "field_1", "field_2", "field_3", "field_4", "field_5", "field_6",
        "field_7", "field_8", "field_9", "field_10", "field_11", "field_12",
        "field_13", "field_14", "field_15"
    ]

    @classmethod
    def user_list(cls, params=None, page=1, size=20, sort="-id"):
        """
        用户基础信息列表服务（优化版）
        改进点：去除未映射的字段
        :param params: 搜索字典参数 dict
        :param page: 当前页码，默认为1
        :param size: 每页大小，默认为20
        :param sort: 排序字段，默认为"-id"
        :return:  data,err
        """
        if params is None:
            params = {}

        # 1. 初始化排序和字段映射
        sort = sort if sort in ["id", "-id", "register_time", "-register_time"] else "-id"

        # 获取有效的拓展字段映射
        field_map_list = list(ExtendField.objects.all().values("field", 'field_index'))
        valid_field_indices = {item['field_index'] for item in field_map_list}

        # 2. 处理搜索参数
        # 创建详情字段别名映射
        detail_alias_dict = {
            field: f'user_detail_info__{field}'
            for field in cls.detail_fields
            if field in valid_field_indices or not field.startswith('field_')
        }

        # 处理参数中的拓展字段
        new_params = {
            item['field_index']: params[item['field']]
            for item in field_map_list
            if item['field'] in params
        }

        # 3. 构建查询条件
        search_params = format_params_handle(
            param_dict={**params, **new_params},
            filter_filed_list=[
                                  "username", "email", "phone", "nickname",
                                  "fullname", "is_using", "user_level"
                              ] + [f for f in cls.detail_fields if
                                   f in valid_field_indices or not f.startswith('field_')],
            alias_dict={
                "username": "username__contains",
                "email": "email__contains",
                "phone": "phone__contains",
                "nickname": "nickname__contains",
                "fullname": "fullname__contains",
                **detail_alias_dict
            },
            is_remove_empty=True,
        )

        # 4. 构建基础查询集
        user_base_set = BaseInfo.objects.filter(is_delete=0)
        user_base_set = user_base_set.extra(
            select={'register_time': 'DATE_FORMAT(register_time, "%%Y-%%m-%%d %%H:%%i:%%s")'}
        ).order_by(sort).annotate(
            user_id=F("id"),
            user_uuid=F("uuid")
        )

        # 5. 应用搜索条件
        if search_params:
            search_filters = [Q(**{key: value}) for key, value in search_params.items()]
            if search_filters:
                combined_filter = search_filters[0]
                for f in search_filters[1:]:
                    combined_filter |= f
                user_base_set = user_base_set.select_related('user_detail_info').filter(combined_filter)

        # 6. 分页处理
        paginator = Paginator(user_base_set, size)
        try:
            page_set = paginator.page(page)
        except EmptyPage:
            page_set = paginator.page(paginator.num_pages)

        # 7. 构建结果数据
        # 7.1 基础信息
        base_list = [{k: getattr(user, k) for k in cls.base_fields} for user in page_set.object_list]

        # 7.2 详情信息（只包含有效字段）
        valid_detail_fields = [
            f for f in cls.detail_fields
            if f in valid_field_indices or not f.startswith('field_')
        ]

        detail_dict = [
            {
                k: getattr(user.user_detail_info, k)
                if hasattr(user, 'user_detail_info') and hasattr(user.user_detail_info, k)
                else None
                for k in valid_detail_fields
            }
            for user in page_set.object_list
        ]

        # 7.3 映射字段名
        field_mapping = {item['field_index']: item['field'] for item in field_map_list}
        detail_list = [
            {field_mapping.get(k, k): v for k, v in detail.items()}
            for detail in detail_dict
        ]

        # 8. 合并结果
        base_join_detail = JJoinList.join(base_list, detail_list, 'id', "user_id")

        return {
            'size': int(size),
            'page': int(page),
            'count': paginator.count,
            'list': base_join_detail
        }, None


import re
import sys
import uuid
from datetime import datetime, timedelta
import json
from logging import getLogger
from pathlib import Path
import string
import random
import jwt
from django.core.cache import cache
from django.db.models import Q, F
from django.forms import model_to_dict
from django.contrib.auth.hashers import make_password, check_password
from django.contrib.sessions.models import Session
from django.db import transaction

from main.settings import BASE_DIR

# TODO 用户模块不希望依赖xj_captcha模块 20241206 by Sieyoo
# from xj_captcha.services.sms_service import SmsService
from ..utils.j_ulid_field import JULIDField

if 'xj_role' in sys.modules:
    from xj_role.services.user_group_service import UserGroupService
# from xj_user.apis.user_platform import UserPlatform
from xj_user.services.user_detail_info_service import DetailInfoService
from xj_user.services.user_relate_service import UserRelateToUserService
from xj_user.services.user_service import UserService
from xj_user.services.user_platform_service import UserPlatformService
# from xj_user.utils.wechat_sign import applet_subscribe_message, subscribe_message
from xj_user.utils.get_short_id import get_short_id
from xj_user.utils.write_to_log import write_to_log
from xj_user.utils.nickname_generate import gen_one_word_digit
from xj_user.utils.wechat import get_openid
from ..models import BaseInfo, Auth, UserSsoToUser, Platform, PlatformsToUsers, UserSsoServe
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict
from ..utils.utility_method import generate_password, replace_placeholders

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
user_main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))
user_module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))

payment_main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
payment_module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))

sub_appid = payment_main_config_dict.wechat_merchant_app_id or payment_module_config_dict.wechat_merchant_app_id or ""
sub_app_secret = payment_main_config_dict.wechat_merchant_app_secret or payment_module_config_dict.wechat_merchant_app_secret or ""
wechat_merchant_name = payment_main_config_dict.wechat_merchant_name or payment_module_config_dict.wechat_merchant_name or ""

# 不明白，为什么小程序appid和密钥要从支付模块中获取，正确应从用户模块获取
# app_id = payment_main_config_dict.app_id or payment_module_config_dict.app_id or ""
# app_secret = payment_main_config_dict.secret or payment_module_config_dict.secret or ""

app_id = user_main_config_dict.app_id or user_module_config_dict.app_id or ""
app_secret = user_main_config_dict.secret or user_module_config_dict.secret or ""

subscription_app_id = payment_main_config_dict.wechat_subscription_app_id or payment_module_config_dict.wechat_subscription_app_id or ""
subscription_app_secret = payment_main_config_dict.wechat_subscription_app_secret or payment_module_config_dict.wechat_subscription_app_secret or ""

app_app_id = payment_main_config_dict.wechat_app_app_id or payment_module_config_dict.wechat_app_app_id or ""
app_app_secret = payment_main_config_dict.wechat_app_app_secret or payment_module_config_dict.wechat_app_app_secret or ""

apple_app_id = payment_main_config_dict.apple_app_id or payment_module_config_dict.apple_app_id or ""

jwt_secret_key = user_main_config_dict.jwt_secret_key or user_module_config_dict.jwt_secret_key or ""
expire_day = user_main_config_dict.expire_day or user_module_config_dict.expire_day or ""
expire_second = user_main_config_dict.expire_second or user_module_config_dict.expire_second or ""
template = user_main_config_dict.template or user_module_config_dict.template or ""

redis_main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="main"))
redis_module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="main"))

redis_host = redis_main_config_dict.redis_host or redis_module_config_dict.redis_host or ""
redis_port = redis_main_config_dict.redis_port or redis_module_config_dict.redis_port or ""
redis_password = redis_main_config_dict.redis_password or redis_module_config_dict.redis_password or ""


class LoginService:
    @classmethod
    def sso_record(cls, user_id, appid):
        sso_set = UserSsoToUser.objects.filter(
            user_id=user_id,
            sso_serve__sso_appid=appid
        ).first()
        if not sso_set:
            return None, "查询用户信息失败"
        return model_to_dict(sso_set), None

    # 生成单点登录记录
    @classmethod
    def sso_verify(cls, sso_serve_id, user_id, appid, is_exist=True, sso_unicode=None, union_code=None):
        """
        生成单点登录记录
        :param sso_serve_id: 单点登录服务ID
        :param user_id: 用户ID
        :param appid: appid
        :param sso_unicode: 单点登录唯一识别码(微信openid)
        :param union_code: union_id
        :return: param_dict
        """
        sso_initialize = UserSsoToUser.objects

        query_dict = {}
        where_dict = {}
        query_dict['is_delete'] = 0
        query_dict['sso_serve_id'] = sso_serve_id

        where_dict['is_delete'] = 0

        if user_id:
            query_dict['user_id'] = user_id
            sso_initialize = sso_initialize.filter(~Q(user_id=user_id))

        # 短信验证码登录和正常登录方式是不会存在openid、appid、union_code
        if sso_unicode:
            query_dict['sso_serve__sso_appid'] = appid
            query_dict['sso_unicode'] = sso_unicode
            where_dict['sso_unicode'] = sso_unicode
            where_dict['sso_serve__sso_appid'] = appid

        if union_code:
            where_dict['union_code'] = union_code
            where_dict.pop("sso_unicode", 100)
            where_dict.pop("sso_serve__sso_appid", 100)

        sso = UserSsoToUser.objects.filter(**query_dict).order_by(
            '-id').first()
        # 创建并查询
        if is_exist:
            # 如果检查不到登录信息则创建
            if not sso:
                sso_data = {
                    "sso_serve_id": sso_serve_id,
                    "user_id": user_id,
                    "sso_unicode": sso_unicode,
                    "union_code": union_code
                }
                create_sso = UserSsoToUser.objects.create(**sso_data)
                if not create_sso:
                    return None, "单点登录写入失败"
            # 创建成功后进行查询 再一次确认创建成功
            sso_set = UserSsoToUser.objects.filter(**query_dict).order_by(
                '-id').first()
            if not sso_set:
                return None, "单点登录用户信息不存在"
            sso_set = model_to_dict(sso_set)
            # 重置老单点登录信息
            # UserSsoToUser.objects.filter(**query_dict).filter(~Q(id=sso_set.get("id", ""))).update(
            #     is_delete=1)
            # 如果没查询到微信unionid 并且传入了如果没查询到微信unionid 则更新
            if not sso_set.get("union_code", "") and union_code:
                UserSsoToUser.objects.filter(
                    Q(is_delete=0) | Q(is_delete__isnull=True),
                    user_id=user_id,
                    sso_serve__sso_appid=appid
                ).update(union_code=union_code)
        else:
            sso_set = sso_initialize.filter(**where_dict).order_by(
                '-id').first()

            if not sso_set:
                return {"user": 0}, "单点记录不存在（微信）"
            sso_set = model_to_dict(sso_set)

        return sso_set, None

    # 生成token
    @classmethod
    def make_token(cls, user_id, user_uuid, account, platform_id=None, platform_code=None):
        # TODO 待删platform_id
        # 生成过期时间
        expire_timestamp = datetime.utcnow() + timedelta(
            days=7,
            seconds=0
        )
        # 返回token
        return jwt.encode(
            payload={'user_id': user_id, 'user_uuid': user_uuid, 'account': account, "platform_code": platform_code,
                     "exp": expire_timestamp},
            key=jwt_secret_key
        )

    # 绑定token
    @classmethod
    def bind_token(cls, user_id, token, is_create=True, password=None):
        if is_create:
            auth = {
                'user_id': user_id,
                'password': make_password(password, None, 'pbkdf2_sha1'),
                'plaintext': password,
                'token': token,
            }
        else:
            auth = {
                'token': token,
            }
        Auth.objects.update_or_create({'user_id': user_id}, **auth)
        auth_set = Auth.objects.filter(
            user_id=user_id,
            token__isnull=False
        ).order_by('-update_time').first()

        if not auth_set:
            return None, "密钥生成失败"
        return auth_set, None

    # ------------------------登录类型判断----------------------------------------------
    @classmethod
    def type_judgment(cls, login_type=None, account=None, phone=None, password=None, platform_code=None, sms_code=None,
                      user_id=None, openid_code=None, phone_code=None, sso_serve_id=None, bind_data={},
                      apple_logo=None):
        """
         登录类型判断
         :param login_type: 登录类型
         :param account: 账户
         :param phone: 手机号
         :param password: 密码
         :param platform_code: 平台码
         :param sms_code: 手机验证码
         :param user_id: 用户ID
         :param openid_code: 微信OPENID 临时code
         :param phone_code: 微信手机code
         :param sso_serve_id: 单点登录id，已弃用 by Sieyoo at 20231219
         :param sso_serve_code: 平台码
         :param bind_data: 绑定数据
         :return: param_dict
        """
        # 初始化
        user_info_set = BaseInfo.objects
        user_info_set = user_info_set.extra(
            select={'register_time': 'DATE_FORMAT(register_time, "%%Y-%%m-%%d %%H:%%i:%%s")'})

        sso_app_id = ""
        openid = ""
        unionid = ""
        wechat = {}
        wx_login_type = ""  # 登录类型

        if login_type == "PASSWORD":  # 账号登录
            wx_login_type = "pwd"

            # 多账号判断，以前的代码，现在不需要了 by Sieyoo at 20231219
            # account = str(account)
            # if re.match(r'(^1[356789]\d{9}$)|(^\+?[78]\d{10}$)', account):
            #     current_user_count = user_info_set.filter(**{"phone": account, "is_delete": 0}).count()
            #     if current_user_count > 1:
            #         return None, {"error": "0", "msg": "您输入的用户存在多账号信息，请通过验证码登录",
            #                       "wechat_data": {"error": "6666"}}

            account_serv, error_text = UserService.check_account(str(account), platform_code)
            if error_text:
                return None, error_text
            user_id = account_serv['user_id']

            auth_serv, auth_error = LoginService.check_login(user_id=user_id, password=password, )

            if auth_error:
                return None, auth_error

            current_user = user_info_set.filter(**{"id": auth_serv['user_id'], "is_delete": 0}).first()

        # TODO 用户模块不希望依赖xj_captcha模块 20241206 by Sieyoo
        # elif login_type == "SMS":  # 短信验证码登录 (比较特殊 支持多用户)
        #     wx_login_type = "sms"
        #     sms, sms_err = SmsService.check_sms({"phone": phone, "sms_code": sms_code})
        #     if sms_err and not user_id:
        #         return None, sms_err
        #     current_user_count = user_info_set.filter(**{"phone": phone, "is_delete": 0}).count()
        #     if current_user_count > 1 and not user_id:
        #         current_user = user_info_set.extra(select={
        #             'avatar': 'SELECT avatar FROM user_detail_info WHERE user_base_info.id = user_detail_info.user_id'})
        #         current_user = current_user.filter(**{"phone": phone, "is_delete": 0}).values("id", "username",
        #                                                                                       "avatar")
        #         return {'token': "", 'user_info': list(current_user)}, None
        #     elif user_id:
        #         current_user = user_info_set.filter(**{"id": user_id, "is_delete": 0}).first()
        #     else:
        #         current_user = user_info_set.filter(**{"phone": phone, "is_delete": 0}).first()

        elif login_type == "WECHAT_APPLET":  # 小程序登录
            wx_login_type = "applet"
            # wechat['appid'] = sub_appid
            # wechat['secret'] = sub_app_secret
            wechat['appid'] = app_id
            wechat['secret'] = app_secret
            wechat['openid_code'] = openid_code
            wechat['phone_code'] = phone_code
            sso_app_id = wechat['appid']
            wechat_user_info, wechat_error = get_openid(login_type, wechat)
            # print('type_judgment wechat_user_info, err: ', wechat_user_info, wechat_error)
            if wechat_error:
                return None, wechat_error
            openid = wechat_user_info.get("openid", "")
            unionid = wechat_user_info.get("unionid", "")
            phone = wechat_user_info.get("phone", "")
            sso_set, sso_err = LoginService.sso_verify(sso_serve_id=sso_serve_id, user_id=None, appid=sso_app_id,
                                                       is_exist=False,
                                                       sso_unicode=openid,
                                                       union_code=unionid)

            if sso_set and not (user_info_set.filter(**{"id": sso_set['user'], "is_delete": 0}).first()):
                current_user = user_info_set.filter(**{"phone": phone, "is_delete": 0}).first()
            elif sso_set:
                current_user = user_info_set.filter(**{"id": sso_set['user'], "is_delete": 0}).first()
            else:
                current_user = user_info_set.filter(**{"phone": phone, "is_delete": 0}).first()

        elif login_type == "WECHAT_WEB":  # 公众号
            wx_login_type = "subscribe"
            wechat['appid'] = subscription_app_id
            wechat['secret'] = subscription_app_secret
            wechat['openid_code'] = openid_code
            sso_app_id = wechat['appid']
            wechat_user_info, err = get_openid(login_type, wechat)
            if err:
                return None, err
            openid = wechat_user_info.get("openid", "")
            unionid = wechat_user_info.get("unionid", "")
            sso_set, sso_err = LoginService.sso_verify(sso_serve_id=sso_serve_id, user_id=None, appid=sso_app_id,
                                                       is_exist=False,
                                                       sso_unicode=openid,
                                                       union_code=unionid)
            current_user = user_info_set.filter(**{"id": sso_set['user'], "is_delete": 0}).first()

        elif login_type == "WECHAT_APP":  # APP
            wx_login_type = "app"
            wechat['appid'] = app_app_id
            wechat['secret'] = app_app_secret
            wechat['openid_code'] = openid_code
            sso_app_id = wechat['appid']
            wechat_user_info, err = get_openid(login_type, wechat)
            if err:
                return None, err
            openid = wechat_user_info.get("openid", "")
            unionid = wechat_user_info.get("unionid", "")
            sso_set, sso_err = LoginService.sso_verify(sso_serve_id=sso_serve_id, user_id=None, appid=sso_app_id,
                                                       is_exist=False,
                                                       sso_unicode=openid,
                                                       union_code=unionid)
            current_user = user_info_set.filter(**{"id": sso_set['user'], "is_delete": 0}).first()
        elif login_type == "APPLE":  # 苹果APP
            wx_login_type = "apple"
            sso_app_id = apple_app_id
            openid = apple_logo
            sso_set, sso_err = LoginService.sso_verify(sso_serve_id=sso_serve_id, user_id=None, appid=sso_app_id,
                                                       is_exist=False,
                                                       sso_unicode=openid,
                                                       union_code=None)
            current_user = user_info_set.filter(**{"id": sso_set['user'], "is_delete": 0}).first()
        else:
            return None, "未支持登录方式"

        user_info = model_to_dict(current_user)
        # print('> LoginService::type_judgment: user_info:', user_info)
        # 由于用户信息中有一个多对多字段'platforms_to_users'，类型为queryset，类名为Platform，且无法被model_to_dict转换，所以先删掉
        # 打印：'platforms_to_users': [<Platform: 金陵村>]
        user_info.pop('platforms_to_users')
        # print('> LoginService::type_judgment: user_info 2:', user_info)

        return {'user_info': user_info, 'phone': phone, 'appid': sso_app_id, 'openid': openid,
                'unionid': unionid, "wx_login_type": wx_login_type}, None

    # -----------------------登录逻辑处理-----------------------------------------------
    @classmethod
    def logical_processing(cls, current_user, phone, sso_serve_id, sso_app_id, openid, unionid, platform_code,
                           other_params, wx_login_type):
        """
          登录逻辑处理
          :param current_user: 用户数据
          :param phone: 手机号
          :param sso_serve_id: 单点登录id
          :param sso_app_id: 单点登录app_id
          :param openid: openid
          :param unionid: unionid
          :param platform_id: 平台id, 安全漏洞, 弃用
          :param platform_code: 平台code
          :return: param_dict
        """

        is_create = True  # 是否是新用户
        if not current_user:  # 如果不存在则为注册
            user_info = LoginService.register_phone_account(phone=phone, platform_code=platform_code)
        else:
            user_info = current_user
            is_create = False
            # 用户存在的时候
            # user_info = model_to_dict(current_user)  #
        # ----------------------------------------------------------------------
        # 多对多模型没法转换成字典 直接在此处踢出
        # user_info.pop("platforms_to_users")

        # 已合并到主代码
        # 检查单点登录信息（查询并创建）针对小程序、公众号、APP，同一微信平台绑定过的绑定
        # if sso_serve_id:
        #     sso_set, sso_err = LoginService.sso_verify(sso_serve_id=sso_serve_id, user_id=user_info.get('id', ""),
        #                                                appid=sso_app_id,
        #                                                is_exist=True,
        #                                                sso_unicode=openid,
        #                                                union_code=unionid)
        #     if sso_err:
        #         return None, sso_err

        # TODO token可以考虑让各个子服务独立获取token，而不是公共生成Token，当然，这样设计好不好有待商考 20230507 by Sieyoo
        token = LoginService.make_token(user_info.get('uuid', ""), user_info.get("username", ""), platform_id,
                                        platform_code)
        password = generate_password(12)
        # 修改用户登录信息，绑定token
        auth_set, auth_err = LoginService.bind_token(user_id=user_info.get('id', ""), token=token,
                                                     is_create=is_create, password=password)
        if auth_err:
            return None, auth_err

        # 代码理解：如果是新用户，注册成功并发送短信通知
        if is_create:
            # 注册成功后发送短信通知
            sms_data = {
                "phone": phone,
                "platform": 'ALi',
                "account": user_info.get("username", ""),
                "pwd": password,
                "type": "PWD"
            }

            if wx_login_type == "applet" or wx_login_type == "sms":
                sms_set, sms_err = SmsService.bid_send_sms(sms_data)
                if sms_err:
                    write_to_log(
                        prefix="首次登录写入用户详细信息异常",
                        content='---首次登录写入用户详细信息异常：' + str(sms_err) + '---',
                        err_obj=sms_err
                    )

            try:
                other_params.setdefault("user_id", user_info.get('id', ""))
                other_params.setdefault("score", "5")  # 用户评分初始化，镖行天下业务逻辑 TODO 后期业务抽离，路程控制
                data, detail_err = DetailInfoService.create_or_update_detail(other_params)
                if detail_err:
                    raise Exception(detail_err)
            except Exception as e:
                write_to_log(
                    prefix="首次登录写入用户详细信息异常",
                    content='---首次登录写入用户详细信息异常：' + str(e) + '---',
                    err_obj=e
                )
            # 用户第一次登录即注册，绑定用户的分组ID
            try:
                group_id = other_params.get("group_id")
                if group_id:
                    data, err = UserGroupService.user_bind_group(user_id=user_info.get('id', ""), group_id=group_id)
                    write_to_log(
                        prefix="group_id:" + str(other_params.get("group_id", "")) + "绑定部门ID异常",
                        content=err
                    )
            except Exception as err:
                write_to_log(
                    prefix="绑定部门ID异常",
                    content="group_id:" + str(other_params.get("group_id", "")),
                    err_obj=err
                )
        # 代码理解：否则是已注册过的用户，同时则将其邀请关系进行绑定？
        else:
            # 绑定用户关系 邀请关系和收益关系
            data, relate_err = UserRelateToUserService.bind_bxtx_relate(params=other_params, user_info=user_info)
            if relate_err:
                write_to_log(
                    prefix='绑定用户关系异常：' + str(relate_err),
                    content='当前用户ID:' + str(user_info.get("id", "")) + '\n detail_params:' + json.dumps(
                        other_params),
                    err_obj=relate_err
                )

        return {'token': "Bearer " + auth_set.token, 'user_info': user_info}, None

    # 验证密码
    @classmethod
    def check_login(cls, user_id: int, user_uuid: str, password, account=None, platform_code=None):
        """
        @param user_id 用户ID
        @param user_uuid 用户UUID
        @param password 用户密码。
        @param account 登陆账号，用于生成Token令牌。
        @param platform_code 平台码，用于区别不同平台独立密码。
        @description 注意：目前密码是明文传输，今后都要改成密文传输
        """
        # 检查平台是否存在
        platform_id = None
        if platform_code:
            platform_set = Platform.objects.filter(platform_code__iexact=platform_code)
            if platform_set.count() == 0:
                # return None, "platform不存在平台名称：" + platform_code
                platform_set = Platform.objects.filter(platform_code="DEFAULT")
            platform_id = platform_set.first().platform_id

        auth_set = Auth.objects.filter(user_id=user_id, password__isnull=False).order_by(
            '-update_time').first()
        if not auth_set:
            return None, "账户尚未开通登录服务：" + account + "(" + str(user_id) + ")"

        # TODO 密码并没有判断是哪个平台配置的密码 20230507 by Sieyoo
        # 判断密码不正确
        is_pass = check_password(password, auth_set.password)
        if not is_pass:
            return None, "密码错误"
        # print(int(Config.getIns().get('xj_user', 'DAY', 7)))
        # print(Config.getIns().get('xj_user', 'JWT_SECRET_KEY', ""))

        # 过期时间
        # int(Config.getIns().get('xj_user', 'DAY', 7))
        # int(Config.getIns().get('xj_user', 'SECOND', 0))
        expire_timestamp = datetime.utcnow() + timedelta(days=7, seconds=0)
        # 为本次登录生成Token并记录
        # todo 漏洞，当用户修改用户名时，并可能导致account失效，是否存用户ID更好
        token = jwt.encode(
            payload={'account': account, 'user_id': user_id, 'user_uuid': user_uuid, 'platform_code': platform_code,
                     'exp': expire_timestamp},
            key=jwt_secret_key)
        # payload = jwt.decode(token, key=Config.getIns().get('xj_user', 'JWT_SECRET_KEY', ""), verify=True, algorithms=["RS256", "HS256"])
        # print("> payload:", payload)
        auth_set.token = token
        auth_set.save()
        return {'token': token}, None

    # 微信登录
    @classmethod
    def check_login_wechat(cls, user_id, phone):
        """
        @param user_id 用户ID
        @param phone 登陆账号，必填，用于生成Token令牌。
        @description 注意：目前密码是明文传输，今后都要改成密文传输
        """
        auth_set = Auth.objects.filter(user_id=user_id, password__isnull=False).order_by('-update_time').first()
        if not auth_set:
            return None, "账户尚未开通登录服务：" + phone + "(" + str(user_id) + ")"

        # 过期时间
        expire_timestamp = datetime.utcnow() + timedelta(days=7, seconds=0)
        # 为本次登录生成Token并记录
        # todo 漏洞，当用户修改用户名时，并可能导致account失效，是否存用户ID更好
        token = jwt.encode(payload={'account': phone, 'user_id': user_id, "exp": expire_timestamp},
                           key=jwt_secret_key)
        # payload = jwt.decode(token, key=Config.getIns().get('xj_user', 'JWT_SECRET_KEY'), verify=True, algorithms=["RS256", "HS256"])
        # print("> payload:", payload)
        auth_set.token = token
        auth_set.save()

        return {'token': token}, None

    # 验证密码
    @classmethod
    def check_login_short(cls, user_id, phone):
        """
        @param user_id 用户ID
        @param phone 登陆账号，必填，用于生成Token令牌。
        @description 注意：目前密码是明文传输，今后都要改成密文传输
        """
        auth_set = Auth.objects.filter(user_id=user_id, password__isnull=False).order_by('-update_time').first()
        if not auth_set:
            return None, "账户尚未开通登录服务：" + phone + "(" + str(user_id) + ")"

        # 过期时间
        expire_timestamp = datetime.utcnow() + timedelta(days=int(expire_day),
                                                         seconds=int(expire_second))
        # 为本次登录生成Token并记录
        # todo 漏洞，当用户修改用户名时，并可能导致account失效，是否存用户ID更好
        token = jwt.encode(payload={'account': phone, 'user_id': user_id, "exp": expire_timestamp},
                           key=jwt_secret_key)
        # payload = jwt.decode(token, key=Config.getIns().get('xj_user', 'JWT_SECRET_KEY'), verify=True, algorithms=["RS256", "HS256"])
        # print("> payload:", payload)
        auth_set.token = token
        auth_set.save()

        return {'token': token}, None

    # 注册手机用户
    @classmethod
    def register_phone_account(cls, phone=None, platform_code=None):
        # uuid_no = uuid.uuid1()
        uuid_no = uuid.uuid4()
        base_info = {
            'username': get_short_id(8),
            'nickname': gen_one_word_digit(),
            'phone': phone,
            'email': '',
            "uuid": str(uuid_no).replace("-", ""),
            "user_type": "PERSON",
            "is_delete": 0
        }
        create_user = BaseInfo.objects.create(**base_info)
        if not create_user:
            return None, "用户注册失败"
        # 注册完成后 重新获取用户信息
        user_info_set = BaseInfo.objects.filter(id=create_user.id).first()
        user_info = model_to_dict(user_info_set)

        # 核查用户后，写入平台码
        if platform_code:
            platform_set = Platform.objects.filter(platform_code=platform_code).first()
            if not platform_set:
                return None, f"平台码不存在：{platform_code}"

            platforms_users = PlatformsToUsers.objects.create(**{
                "platform_id": platform_set.id,
                "user_id": user_info.get('id', "")
            })
            if not platforms_users:
                return None, "平台写入失败"

        return user_info, None

    # 绑定手机号
    @classmethod
    def bind_phone(cls, user_id, phone):
        """
        @param user_id 用户ID
        @param phone 手机号
        """
        if not phone:
            return None, "手机号不能为空1"
        mate = re.match(r'(^1[356789]\d{9}$)|(^\+?[78]\d{10}$)', phone)
        if not mate:
            return None, "手机号格式不正确"

        binding_verification, err = LoginService.phone_binding_verification(phone)
        if err:
            return None, err
        bind_user = BaseInfo.objects.filter(id=user_id).update(**{
            "phone": phone
        })
        if not bind_user:
            return None, "用户绑定手机号失败，请联系管理员处理"
        return {"user_id": user_id, "phone": phone}, None

    # 手机绑定验证 一个手机号最多绑定5个账户
    @classmethod
    def phone_binding_verification(cls, phone):
        """
        @param phone 手机号
        """
        if not phone:
            return None, "手机号不能为空2"
        mate = re.match(r'(^1[356789]\d{9}$)|(^\+?[78]\d{10}$)', phone)
        if not mate:
            return None, "手机号格式不正确"
        bind_user = BaseInfo.objects.filter(**{"phone": phone, "is_delete": 0}).count()
        if bind_user >= 1:
            # return None, "同一手机号最多绑定5个账户,已绑定" + str(bind_user) + "个"
            return None, "该手机号已被其他账号绑定"
        return {"phone": phone, "bind_num": bind_user}, None

    # 二次授权
    @classmethod
    def secondary_authorization(cls, params):
        """
        @param user_id 用户ID
        @param code 微信code
        """
        user_id = params.get("user_id", "")
        sso_serve_id = params.get("sso_serve_id", "")
        login_type = params.get("login_type", "")
        code = params.get("code", "")
        sso_code = params.get("sso_code", "")
        if not sso_serve_id and sso_code:
            sso_serve = UserSsoServe.objects.filter(sso_code=sso_code).first()
            sso_serve_id = sso_serve.id
        wechat = {}

        if login_type == "WECHAT_APPLET":  # 小程序
            wechat['appid'] = sub_appid
            wechat['secret'] = sub_app_secret
            wechat['code'] = code
            sso_app_id = wechat['appid']
            wechat_user_info, err = get_openid(login_type, wechat)
            if err:
                return None, err
            openid = wechat_user_info.get("openid", "")
            unionid = wechat_user_info.get("unionid", "")

        elif login_type == "WECHAT_WEB":  # 公众号
            wechat['appid'] = subscription_app_id
            wechat['secret'] = subscription_app_secret
            wechat['code'] = code
            sso_app_id = wechat['appid']
            wechat_user_info, err = get_openid(login_type, wechat)
            if err:
                return None, err
            openid = wechat_user_info.get("openid", "")
            unionid = wechat_user_info.get("unionid", "")

        elif login_type == "WECHAT_APP":  # APP
            wechat['appid'] = app_app_id
            wechat['secret'] = app_app_secret
            wechat['code'] = code
            sso_app_id = wechat['appid']
            wechat_user_info, err = get_openid(login_type, wechat)
            if err:
                return None, err
            openid = wechat_user_info.get("openid", "")
            unionid = wechat_user_info.get("unionid", "")

        sso_exist, sso_exist_err = LoginService.sso_verify(sso_serve_id=sso_serve_id, user_id=user_id,
                                                           appid=sso_app_id,
                                                           is_exist=False,
                                                           sso_unicode=openid,
                                                           union_code=None)

        if sso_exist_err:
            sso_set, sso_err = LoginService.sso_verify(sso_serve_id=sso_serve_id, user_id=user_id, appid=sso_app_id,
                                                       is_exist=True,
                                                       sso_unicode=openid,
                                                       union_code=unionid)
        else:
            return None, "微信已被绑定"

        if sso_err:
            return None, "绑定失败"
        return sso_set, None

    @classmethod
    def wechat_unbinding(cls, params):
        user_id = params.get("user_id", "")
        login_type = params.get("login_type", "")
        if login_type == "WECHAT_APPLET":  # 小程序
            UserSsoToUser.objects.filter(
                Q(is_delete=0) | Q(is_delete__isnull=True),
                user_id=user_id,
                sso_serve__sso_appid=sub_appid
            ).update(is_delete=1)
        elif login_type == "WECHAT_WEB":  # 公众号
            UserSsoToUser.objects.filter(
                Q(is_delete=0) | Q(is_delete__isnull=True),
                user_id=user_id,
                sso_serve__sso_appid=subscription_app_id
            ).update(is_delete=1)
        elif login_type == "WECHAT_APP":  # APP
            UserSsoToUser.objects.filter(
                Q(is_delete=0) | Q(is_delete__isnull=True),
                user_id=user_id,
                sso_serve__sso_appid=app_app_id
            ).update(is_delete=1)

        return None, None

    # 注册信息写入
    @classmethod
    def register(cls, username: str = None, phone: int = None, email: str = None, password: str = None, nickname: str = None,
                 platform_code: str = None):

        # 判断平台存在
        platform_id = None
        if platform_code:
            platform, err = UserPlatformService.get_detail(platform_code)
            if err:
                return None, err
            platform_id = platform.get('id')

        exists = BaseInfo.objects.filter(username=username).exists()
        if exists:
            return None, "账号已存在"

        # 创建并写用户
        new_user = BaseInfo(
            username=username,
            phone=phone,
            email=email,
            nickname=nickname if nickname else gen_one_word_digit(),
            uuid=JULIDField.get_u12(),
            user_type="PERSON"
        )
        new_user.save()
        if not new_user:
            return None, f"用户注册失败({username})"

        # 判断并写平台多对多用户表
        if platform_id:
            platforms_users = PlatformsToUsers(
                platform_id=platform_id,
                user_id=new_user.id,
            )
            platforms_users.save()
            if not platforms_users:
                return None, "平台写入失败"

        # 生成用户令牌
        token = LoginService.make_token(user_uuid=new_user.uuid, account=new_user.username, platform_code=platform_code)

        # 写入登录密码。
        auth_set, auth_err = LoginService.bind_token(user_id=new_user.id, token=token, is_create=True, password=password)
        if auth_err:
            return None, auth_err

        # return {'token': "Bearer " + auth_set.token, 'user_info': model_to_dict(new_user)}, None
        return {'token': "Bearer " + auth_set.token}, None
